"""Module for the Round class."""

from collections import UserList
from pathlib import Path
from random import shuffle
from typing import List, Optional

from pubquiz.question import Question
from pubquiz.slides import header_slide


class Round(UserList):
    """Class representing a round in a pub quiz."""

    def __init__(
        self,
        title,
        description="",
        questions: Optional[List[Question]] = None,
        solve_in_own_time: bool = False,
        randomize: bool = False,
        sheets: Optional[Path] = None,
    ):
        """Initialize the round."""
        questions = questions or []
        if randomize:
            shuffle(questions)
        super().__init__(questions)
        self.title = title
        self.description = description
        self.solve_in_own_time = solve_in_own_time
        self.randomize = randomize
        self.sheets = sheets

    def __repr__(self):
        return f"Round(title={self.title})"

    @classmethod
    def from_dict(cls, dct):
        """Create a round object from a dictionary."""
        questions = dct.pop("questions", [])
        return cls(**dct, questions=[Question.from_dict(q) for q in questions])

    def _sheets_content(self, with_answers: bool = True) -> List[str]:
        """Generate the LaTeX code for the body of the sheet for this round, either with or without the answers."""
        if self.sheets:
            return [r"\input{" + str(self.sheets) + "}"]

        lines = []
        if with_answers:
            lines += [r"\large", r"\begin{enumerate}"]
            lines += [r"\item " + str(q) for q in self]
            lines += [r"\end{enumerate}", r"\LARGE"]
        else:
            if self.solve_in_own_time:
                lines += [r"\large"]
            else:
                lines += [r"\Huge"]
            lines += [r"\begin{enumerate}"]
            if self.solve_in_own_time:
                # Show the questions
                lines += [rf"\item {q.question}" for q in self]
            else:
                lines += [r"\item" for _ in self]
            lines += [r"\end{enumerate}", ""]
        return lines

    def to_sheets(self, with_answers=True, index=1) -> List[str]:
        """Generate the LaTeX code for the quiz sheets."""
        header = self.title if ":" in self.title else f"Round {index}: {self.title}"
        lines = [r"\newpage", r"\begin{center}", r"\Huge", header, r"\end{center}"]

        lines += [r"\large"]

        if len(self.description) > 0:
            lines += [self.description, ""]

        lines += self._sheets_content(with_answers=with_answers)
        return lines

    def _slides_content(self, with_answers: bool = True) -> List[str]:
        """Generate the LaTeX code for the slides, either with or without the answers."""
        lines = []
        for iq, q in enumerate(self):
            lines.append(q.to_slide(index=iq + 1, with_answer=with_answers))
        return lines

    def to_slides(self, index=1) -> List[str]:
        """Generate the LaTeX code for the slides.

        Includes headers and (possibly) first all the questions without and then with the answers

        :param index: the index of the round
        :type index: int

        :returns: a list containing the lines of latex code for the slides
        """
        # Round header
        if ":" not in self.title:
            heading = f"Round {index}: {self.title}"
        else:
            heading = self.title
        lines = header_slide(heading)

        if not self.solve_in_own_time:
            # Questions without answers
            lines += self._slides_content(with_answers=False)

            # Answer header
            lines += header_slide("Answers")

        # Questions with answers
        lines += self._slides_content(with_answers=True)

        return lines



"""Module containing the Quiz class."""

import shutil
from collections import UserList
from pathlib import Path
from typing import List, Optional

from yaml import safe_load

from pubquiz.latex_templates import path as latex_templates_path
from pubquiz.round import Round


class Quiz(UserList):
    """Class representing a pub quiz."""

    def __init__(
        self, title, author: str, date: str = r"\today", rounds: Optional[List[Round]] = None
    ):
        """Initialize the quiz."""
        rounds = rounds or []
        super().__init__(rounds)
        self.title = title
        self.author = author
        self.date = date

    def __repr__(self) -> str:
        return f"Quiz(title={self.title}, rounds=[{', '.join([r.title for r in self])}])"

    @classmethod
    def from_dict(cls, dct):
        """Create a quiz object from a dictionary."""
        rounds = dct.pop("rounds", [])
        return cls(**dct, rounds=[Round.from_dict(r) for r in rounds])

    @classmethod
    def from_yaml(cls, filename: Path):
        """Create a quiz object from a yaml file."""
        with open(filename, "r") as f:
            dct = safe_load(f)
        return cls.from_dict(dct)

    def to_sheets(self, with_answers=False) -> str:
        """
        Generate the latex code for the quiz sheets.

        :param with_answers: if True, the answers to the questions will be included in the sheets.
        :type with_answers: bool

        :returns: a list of strings containing the latex code for the quiz sheets
        """
        # Make sure we have sheets_header.tex in the current directory
        if not Path("sheets_header.tex").exists():
            shutil.copy(latex_templates_path / "sheets_header.tex", ".")
            print(
                "Generating a default sheets_header.tex file. Please edit this file to suit your needs."
            )

        # N.B. will not do picture and puzzle rounds, these must be contained in pictures.tex and puzzles.tex
        titlepage = (
            [
                r"\centering",
                r"\Huge",
                self.title,
                r"\vspace{2cm}",
                r"",
                r"\LARGE",
                r"Team Name: \underline{\hphantom{XXXXXXXXXXXXXXXXXXXXXXXXXX}}",
                r"",
                r"\vspace{3cm}",
                r"",
                r"\LARGE",
                r"\begin{tabular}{ll}",
                r"\hline",
                r"Round & Score \\",
                r"\hline",
            ]
            + [r.title + r" & \\" for r in self]
            + [
                r"TOTAL \\",
                r"\hline",
                r"\end{tabular}",
                r"\thispagestyle{empty}",
                r"\Huge",
            ]
        )

        # Header
        lines = [r"\input{sheets_header}"]
        if not with_answers:
            lines += [r"\rhead{\huge \fbox{\parbox{3.5cm}{Score}}}"]
        lines += [r"\begin{document}"]

        if not with_answers:
            lines += titlepage

        # Standard rounds
        for i, r in enumerate(self):
            lines += r.to_sheets(with_answers=with_answers, index=i + 1)

        # Footer
        lines += [r"\end{document}"]

        return "\n".join(lines)

    def to_slides(self) -> str:
        """Generate the latex code for the quiz slides."""
        # Ensure we have the header and preamble
        if not Path("slides_header.tex").exists():
            shutil.copy(latex_templates_path / "slides_header.tex", ".")
            print(
                "Generating a default slides_header.tex file. Please edit this file to suit your needs."
            )
        if not Path("photo.png").exists():
            shutil.copy(latex_templates_path / "photo.png", ".")
            print(
                "Generating a default photo.png file to use in the title slide. Please replace this file to "
                "suit your needs."
            )
        if not Path("slides_preamble.tex").exists():
            shutil.copy(latex_templates_path / "slides_preamble.tex", ".")
            print(
                "Generating a default slides_preamble.tex file. Please edit this file to suit your needs."
            )

        # Header
        lines = [
            r"\input{slides_header}",
            r"\title{" + self.title + "}",
            r"\author{" + self.author + "}",
        ]
        date = self.date or r"\today"
        lines += [r"\date{" + date + "}"]
        lines += [r"\begin{document}", r"\frame{\titlepage}", r"\include{slides_preamble}"]

        # Loop over rounds and generate slides
        for i, r in enumerate(self):
            lines += r.to_slides(index=i + 1)

        # Footer
        lines += [r"\end{document}"]

        return "\n".join(lines)



"""Module containing useful functions for generating LaTeX slides."""


def header_slide(header):
    """Generate a generic header slide with the heading 'header'."""
    return [r"\begin{frame}", r"\begin{center}", r"\Huge", header, r"\end{center}", r"\end{frame}"]



"""Testing the Quiz class."""

from pathlib import Path

from pubquiz import Quiz


def test_from_yaml():
    """Test the :classmethod:Quiz.from_yaml() classmethod."""
    Quiz.from_yaml(Path(__file__).parents[1] / "docs/source/example_quiz.yaml")



# -*- coding: utf-8 -*-

"""Command line interface for :mod:`pubquiz`.

Why does this file exist, and why not put this in ``__main__``? You might be tempted to import things from ``__main__``
later, but that will cause problems--the code will get executed twice:

- When you run ``python3 -m pubquiz`` python will execute``__main__.py`` as a script.
  That means there won't be any ``pubquiz.__main__`` in ``sys.modules``.
- When you import __main__ it will get executed again (as a module) because
  there's no ``pubquiz.__main__`` in ``sys.modules``.

.. seealso:: https://click.palletsprojects.com/en/8.1.x/setuptools/#setuptools-integration
"""

import logging
import subprocess

import click

from pubquiz import Quiz

__all__ = [
    "main",
]

logger = logging.getLogger(__name__)


@click.group()
@click.version_option()
def main():
    """CLI for pubquiz."""


valid_outputs = ["sheets", "slides"]


# Make a pub quiz from a yaml file
@main.command()
@click.argument("yaml_file", type=click.Path(exists=True))
@click.argument("output", type=click.Choice(valid_outputs + ["all"]), default="all")
@click.option("--no-compile", is_flag=True, default=False, help="Do not compile the output files.")
def make(yaml_file, output, no_compile):
    """Make a pub quiz from a yaml file."""
    if output == "all":
        outputs = valid_outputs
    else:
        outputs = [output]

    quiz = Quiz.from_yaml(yaml_file)

    for o in outputs:
        if o == "sheets":
            string = quiz.to_sheets()
        elif o == "slides":
            string = quiz.to_slides()
        with open(f"{o}.tex", "w") as f:
            f.write(string)

        if not no_compile:
            proc = subprocess.run(
                ["pdflatex", "-interaction=nonstopmode", f"{o}.tex"], stdout=subprocess.DEVNULL
            )
            if proc.returncode != 0:
                logger.error(
                    f"'pdflatex {o}.tex' returned non-zero exit code. Try running pdflatex manually to "
                    "see what went wrong."
                )


if __name__ == "__main__":
    main()



"""Module defining the Question class."""

from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional


@dataclass
class Question:
    """Class representing a question in a pub quiz."""

    question: str
    answer: str
    slide: Optional[List[str]] = None
    question_pic: Optional[Path] = None
    question_pic_height: float = 0.6
    answer_pic: Optional[Path] = None
    answer_pic_height: float = 0.6

    def __post_init__(self):
        if self.question and Path(self.question).exists():
            self.question = r"\input{" + self.question + "}"
        if self.slide and Path(self.slide).exists():
            self.slide = r"\input{" + self.slide + "}"

    def __repr__(self):
        return self.question + " (" + self.answer + ")"

    @classmethod
    def from_dict(cls, dct):
        """Create a question object from a dictionary."""
        return cls(**dct)

    def to_slide(self, index, with_answer=False):
        """Generate the LaTeX code for a slide presenting this question."""
        # Allow manual override of as_slide(), to allow for more complex slides to be generated by hand
        if self.slide:
            return self.slide

        lines = [r"\begin{frame}", r"\begin{center}", r"\Large", f"{index}. {self.question}"]
        if self.question_pic:
            pic = r"\vspace{0.5em}"
            pic += (
                r"\includegraphics[height="
                + str(self.question_pic_height)
                + r"\paperheight]{"
                + self.question_pic
                + "}"
            )
            if with_answer and self.answer_pic:
                lines += [r"\\", r"\only<1>{" + pic + "}"]
            else:
                lines += [r"\\", pic]
        if with_answer:
            if self.answer_pic:
                pic = r"\vspace{0.5em}"
                pic += (
                    r"\includegraphics[height="
                    + str(self.answer_pic_height)
                    + r"\paperheight]{"
                    + self.answer_pic
                    + "}"
                )
                if self.question_pic:
                    lines += [r"\only<2>{" + pic + "}"]
                else:
                    lines += [r"\\", r"\onslide<2>{" + pic + "}"]
            if r"\input" not in lines[-1]:
                lines += [r"\\"]
            if self.answer:
                lines += [r"\onslide<2->{\vspace{1em}\textit{" + self.answer + "}}"]
        lines += [r"\end{center}", r"\end{frame}"]

        return "\n".join(lines)
